﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using System.ServiceModel;
using HeuristicLab.Services.Optimization.ControllerService;
using HeuristicLab.Services.Optimization.ControllerService.Model;
using System.ServiceModel.Description;
using System.Web.Security;
using HeuristicLab.Services.Optimization.Web.Models;

namespace HeuristicLab.Services.Optimization.Web.Controllers
{
    [Authorize(Roles="Web User")]
    public class OptimizationController : Controller
    {
      public const string DEFAULT_CONTROLLER_ENDPOINT = "WSHttpBinding_IControllerService";

      public delegate T ControllerServiceDelegate<T>(IControllerService service);
      public delegate void ControllerServiceDelegate(IControllerService service);

      public T withControllerService<T>(ControllerServiceDelegate<T> del) {
        if (Session["pw"] == null)
          Response.Redirect("/Account/Logon");

        using (var cf = new ChannelFactory<IControllerService>(OptimizationController.DEFAULT_CONTROLLER_ENDPOINT)) {
          var credentialBehaviour = cf.Endpoint.Behaviors.Find<ClientCredentials>();
          credentialBehaviour.UserName.UserName = Membership.GetUser().UserName;
          credentialBehaviour.UserName.Password = Session["pw"] as string;
          var controllerProxy = cf.CreateChannel();
          return del(controllerProxy);
        }
      }

      public void withControllerService(ControllerServiceDelegate del) {
        if (Session["pw"] == null)
          Response.Redirect("/Account/Logon");

        using (var cf = new ChannelFactory<IControllerService>(OptimizationController.DEFAULT_CONTROLLER_ENDPOINT)) {          
          var credentialBehaviour = cf.Endpoint.Behaviors.Find<ClientCredentials>();
          credentialBehaviour.UserName.UserName = Membership.GetUser().UserName;
          credentialBehaviour.UserName.Password = Session["pw"] as string;          
          var controllerProxy = cf.CreateChannel();
          del(controllerProxy);
        }
      }
        //
        // GET: /Optimization/        

        public ActionResult Index()
        {
          var optModel = withControllerService<OptimizationModel>((service) => {
            OptimizationModel model = new OptimizationModel();
            model.Scenarios = service.GetOptimizationScenarios();
            User u = new User() { Username = Membership.GetUser().UserName, Password = Session["pw"] as string };
            model.Jobs = service.GetJobs(u);
            return model;
          });
          return View(optModel);
        }

        public ActionResult ProblemParameters(string scenario) {
          var optScenario = withControllerService<OptimizationScenario>((service) => {
            return service.GetOptimizationScenarioByName(scenario);
          });
          Session["scenario"] = optScenario;
          return View(optScenario.InputParameters);
        }

        public ActionResult JobDetails(string jobId) {
          Job job = withControllerService<Job>((service) => {
            User u = new User() { Username = Membership.GetUser().UserName, Password = Session["pw"] as string };
            return service.GetJob(u, jobId);
          });
          var runs = withControllerService<IList<Run>>((service) => {
            User u = new User() { Username = Membership.GetUser().UserName, Password = Session["pw"] as string };
            return service.GetJobResults(u, jobId);
          });
          JobDetailsModel jdm = new JobDetailsModel() { Job = job, Runs = runs };
          Session["jobDetails"] = jdm;
          return View(jdm);
        }

        [HttpPost]
        public ActionResult JobDetails(Job job) {
          withControllerService((service) => {
            User u = new User() { Username = Membership.GetUser().UserName, Password = Session["pw"] as string };
            service.DeleteJob(u, job.Id);
          });
          return RedirectToAction("Index");
        }

        [HttpPost]
        public ActionResult ProblemParameters(InputParameters parameters) {
          if (ModelState.IsValid) {
            return RedirectToAction("AlgorithmParameters", parameters);
          } 
          return View(parameters);        
        }

        public ActionResult AlgorithmParameters() {
          return View((Session["scenario"] as OptimizationScenario).AlgorithmParameters);
        }

        [HttpPost]
        public ActionResult AlgorithmParameters(AlgorithmParameters parameters) {
          if (ModelState.IsValid) {
            return RedirectToAction("ScheduleJob");
          }
          return View(parameters);
        }

        public ActionResult ScheduleJob() {
          return View(Session["scenario"]);
        }

        [HttpPost]
        public ActionResult ScheduleJob(OptimizationScenario scenario) {
          using (var cf = new ChannelFactory<IControllerService>(OptimizationController.DEFAULT_CONTROLLER_ENDPOINT)) {
            var credentialBehaviour = cf.Endpoint.Behaviors.Find<ClientCredentials>();
            credentialBehaviour.UserName.UserName = Membership.GetUser().UserName;
            credentialBehaviour.UserName.Password = Session["pw"] as string;
            var controllerProxy = cf.CreateChannel();
            User u = new User() { Username = Membership.GetUser().UserName, Password = Session["pw"] as string };
            controllerProxy.ScheduleOptimizationScenario(u, Session["scenario"] as OptimizationScenario);
          }
          return RedirectToAction("Index");
        }
    }
}
